@@ -137,7 +137,7 @@ class CCS(Calculator):
     >>> To be added, Jolla.
     """
 
-    implemented_properties = {"stress", "energy", "forces"}
+    implemented_properties = {"energy", "forces"} # stress
 
     def __init__(
         self, CCS_params=None, charge=None, q=None, charge_scaling=False, **kwargs
@@ -241,6 +241,7 @@ class CCS(Calculator):
             energy = energy + ewa.total_energy
             forces = forces + ewa.forces
 
+
         self.results["energy"] = energy
         self.results["free_energy"] = energy
         self.results["forces"] = forces
